/*
 * Test dataspace operations.
 *
 * Copyright (C) 2020, 2021, 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <systypes/fcntl.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <fsclient/client.h>



int main(int argc, char *argv[])
{
  if (argc < 2)
  {
    printf("Need a filename and optional repetition.\n");
    return 1;
  }

  /* Obtain filename and access parameters. */

  char *filename = argv[1];
  int repetition = argc > 2 ? atoi(argv[2]) : 10;
  file_t *file;

  file = client_open(filename, O_RDWR);

  if (!client_opened(file))
  {
    client_close(file);
    printf("Could not obtain file.\n");
    return 1;
  }

  /* Copy the sampled data to another file region. */

  offset_t size = file->size;
  offset_t new_size = size * (repetition + 1);

  char buffer[size];
  offset_t nread = client_read(file, buffer, size);

  if (nread != size)
  {
    printf("Could not read entire file: %ld out of %ld bytes.\n", nread, size);
    return 1;
  }

  printf("File contents...\n");

  fwrite(buffer, sizeof(char), nread, stdout);

  printf("Copy %ld bytes to end...\n", nread);

  for (int times = 0; times < repetition; times++)
  {
    printf("Copy #%d...\n", times);

    offset_t nwritten = client_write(file, buffer, nread);

    if (nwritten != nread)
    {
      printf("Could not write file section: %ld instead of %ld bytes.\n",
             nwritten, nread);
      return 1;
    }
  }

  printf("File now has size %ld.\n", file->size);
  printf("File has expected size %ld: %s\n", new_size, new_size == file->size ? "True" : "False");

  printf("Seek to start...\n");

  client_seek(file, 0, SEEK_SET);

  printf("At mapped region from %ld to %ld with data at %ld to %ld.\n",
         file->start_pos, file->end_pos, file->data_current, file->data_end);

  printf("File contents...\n");

  offset_t position = 0;

  while ((nread = client_read(file, buffer, size)) > 0)
  {
    if ((nread != size) && (nread != file->size - position))
    {
      printf("Could not read file section: %ld instead of %ld or remaining %ld bytes.\n",
             nread, size, file->size - position);
      return 1;
    }

    fwrite(buffer, sizeof(char), nread, stdout);
    position += nread;
  }

  printf("File shown in its entirety: %s\n", position == file->size ? "True" : "False");
  printf("End of test.\n");
  return 0;
}

// vim: tabstop=2 expandtab shiftwidth=2
